// TemplateBlock.cpp: implementation of the CTemplateBlock class.
//

#include "stdafx.h"

#include "Template.h"
#include "TemplateBlock.h"
#include "TemplateBlockDlg.h"
#include "SDI/SDIInterface.h"
#include "Utils/Report.h"
#include "Utils/ResourceKey.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// CTemplateBlock Class
//
CTemplateBlock::CTemplateBlock()
   : TModule(_T("TemplateBlock"), TemplateSLL.hResource)
{
   // TODO: Add the block's properties to the properties list 
   // by calling AddProperty (e.g., AddProperty(_T("prop_name"), m_prop)).
   // This mechanism is what facilitates the save and restore of 
   // properties to and from setup files.

   // Add analog I/O properties.
   AddProperty(_T("AIOChannel"), m_propAIOChannel = 0);
   AddProperty(_T("AIOModule"), m_propAIOModule = 1);

   // Add digital I/O properties.
   AddProperty(_T("DIOChannel"), m_propDIOChannel = 0);
   AddProperty(_T("DIOModule"), m_propDIOModule = 1);

   // Add SDI properties.
   AddProperty(_T("SDIAddressIdx"), m_propSDIAddressIdx = 0);
}


CTemplateBlock::~CTemplateBlock()
{
   ClearChannelsInuse();
}


CString CTemplateBlock::InputName(int Point)
{
   // Return the name of the input associated with Point.
   switch (Point)
   {
      case 2:
         return _T("In1");
      default:
         return _T("");
   }
}


CString CTemplateBlock::OutputName(int Point)
{
   // Return the name of the output associated with Point.
   switch (Point)
   {
      case 2:
         return _T("Out1");
      default:
         return _T("");
   }
}


bool CTemplateBlock::InputActive(int Point)
{
   // Return true if the input associated with Point is one we use.
   // Return false, otherwise.
   if (2 == Point)
      return true;
   else 
      return false;
}


bool CTemplateBlock::OutputActive(int Point)
{
   // Return true if the output associated with Point is one we use.
   // Return false, otherwise.
   if (2 == Point)
      return true;
   else 
      return false;
}

   
bool CTemplateBlock::ShowProperties(CWnd* pParent)
{
   // This method is called whenever the properties dialog of this block 
   // should be shown. An instance of TResourceKey is created and attached 
   // to TemplateSLL so that references to resources are assumed to be references 
   // to resources stored in this module. Return true only on IDOK.
   TResourceKey key(TemplateSLL);
   CTemplateBlockDlg dlg(pParent);

   // TODO: Initialize dialog data before showing it.

   dlg.m_iAIOChannel = m_propAIOChannel;
   dlg.m_iAIOModule = m_propAIOModule;
   dlg.m_iDIOChannel = m_propDIOChannel;
   dlg.m_iDIOModule = m_propDIOModule;
   dlg.m_iSDIAddressIdx = m_propSDIAddressIdx;
   if (dlg.DoModal() == IDOK)
   {
      // TODO: Incorporate data from dialog.

      ClearChannelsInuse();
      m_propAIOChannel = dlg.m_iAIOChannel;
      m_propAIOModule = dlg.m_iAIOModule;
      m_propDIOChannel = dlg.m_iDIOChannel;
      m_propDIOModule = dlg.m_iDIOModule;
      SetChannelsInuse();

      m_propSDIAddressIdx = dlg.m_iSDIAddressIdx;
      return true;
   }
   return false;
}


void CTemplateBlock::Initialize()
{
   // This method is called when the user presses Start. 

   // Initialize m_Out1Data.
   m_Out1Data.TimeScheduled = TTime::GetCurrentTime();
   m_Out1Data.TimeActual = TTime::GetCurrentTime();
   m_Out1Data.SensorID = Function.AsCString();
   m_Out1Data.Quality = CSensorData::UNDEFINED;
   m_Out1Data.Data = 0;
   m_Out1Data.Units = _T("");

   // Get handle to analog device.
   AnalogIO* pAnalogIO = Engine.IOModList.GetAnalogIO(m_propAIOModule);
   if (!pAnalogIO)
   {
      Report.Error(_T("%s - Initialize: Failed to get analog device."), 
            Function.AsCString());
      return;
   }

   // Get handle to digital device.
   DigitalIO* pDigIO = Engine.IOModList.GetDigitalIO(m_propDIOModule);
   if (!pDigIO)
   {
      Report.Error(_T("%s - Initialize: Failed to get digital device."), 
            Function.AsCString());
      return;
   }

   // TODO: Configure I/O module(s) as necessary using the handle(s) created 
   // above. This typically involves calls into the I2C API.

   // TODO: Configure SDI device as necessary. This typically involves calls 
   // into the SDI API. 

   // TODO: Add any processing that should occur after the user presses 
   // the Start recording button.

}


void CTemplateBlock::Execute(TTime tScheduled)
{
   // This method is called as part of a push or pull of data through the 
   // system initiated by Engine. This is where this module performs the 
   // work it was created to do, whether that be take a measurement, perform 
   // a calculation, etc.

   // Initialize intermediate Out1 data quality to bad.
   CSensorData Out1Data = m_Out1Data;
   Out1Data.TimeScheduled = tScheduled;
   Out1Data.TimeActual = TTime::GetCurrentTime();
   Out1Data.Quality = CSensorData::BAD;
   Out1Data.Data = 0.0;

   // Get handle to analog device.
   AnalogIO* pAnalogIO = Engine.IOModList.GetAnalogIO(m_propAIOModule);
   if (!pAnalogIO)
   {
      Report.Error(_T("%s - Execute: Failed to get analog device."), Function.AsCString());
      return;
   }

   // Get handle to digital device.
   DigitalIO* pDigIO = Engine.IOModList.GetDigitalIO(m_propDIOModule);
   if (!pDigIO)
   {
      Report.Error(_T("%s - Execute: Failed to get digital device."), Function.AsCString());
      return;
   }

   // Get SDI address.
   CString strAddress = (CString)szSDIAddrSet[m_propSDIAddressIdx.AsInteger()];

   // TODO: Add code to perform module's function. On successful performance of 
   // that function, be sure to set the quality flag of the intermediate data to 
   // good.

   // Buffer the output data.
   LockData();
   m_Out1Data = Out1Data;
   UnlockData();
}


void CTemplateBlock::GetData(int nPt, CSensorData& data)
{
   switch (nPt)
   {
      case 2:
         data = m_Out1Data;
         break;
   }
}


void CTemplateBlock::SetData(int nPt, const CSensorData& data)
{
   switch (nPt)
   {
      case 2:
         m_Out1Data = data;
         break;
   }
}


bool CTemplateBlock::GetIOAddrAsString(CString& string)
{
   // This method is called to obtain a string representation of the "I/O 
   // Address" used by this module.

   /*
   // TODO: This module has more than one "I/O Address" (the address displayed 
   // on the ViewSensors page) since it either uses multiple I/O module types, 
   // or uses one or more I/O module types in conjunction with SDI. A default 
   // I/O Address has been coded for you. You may want to change this default 
   // to return another address. Examples of the other address types follow:

   // Return SDI address.
   string = (CString)_T("SDI ") + (CString)szSDIAddrSet[m_propSDIAddressIdx.AsInteger()];
   return true;

   // Return digital I/O address.
   return Engine.IOModList.GetIOAddrAsString(DIGITAL, m_propDIOModule, 
         m_propDIOChannel.AsInteger(), string);
   */
   
   // Return analog I/O address.
   return Engine.IOModList.GetIOAddrAsString(ANALOG, m_propAIOModule, 
         m_propAIOChannel.AsInteger(), string);
}


void CTemplateBlock::AssignDefaultChannel()
{
   // This method is called when a module of this type is added to the setup. 
   // It looks for the first available module and channel combination and 
   // assigns it to the local channel and module variables.
   Engine.IOModList.FindChannel(ANALOG, m_propAIOModule.AsIntRef(), 
         m_propAIOChannel.AsIntRef());
   Engine.IOModList.FindChannel(DIGITAL, m_propDIOModule.AsIntRef(), 
         m_propDIOChannel.AsIntRef());
}


CIODeviceType CTemplateBlock::GetDeviceType()
{
   // TODO: This module uses more than one I/O module. The analog property 
   // has been selected by default but you may want to change it to digital.
   return ANALOG;
}


TProperty* CTemplateBlock::GetModuleProperty()
{
   // TODO: This module uses more than one I/O module. The analog property 
   // has been selected by default but you may want to change it to digital.
   return &m_propAIOModule;
}


TProperty* CTemplateBlock::GetChannelProperty()
{
   // TODO: This module uses more than one I/O module. The analog property 
   // has been selected by default but you may want to change it to digital.
   return &m_propAIOChannel;
}


void CTemplateBlock::ClearChannelsInuse()
{
   Engine.IOModList.ClearChannelInUse(ANALOG, m_propAIOModule, m_propAIOChannel);
   Engine.IOModList.ClearChannelInUse(DIGITAL, m_propDIOModule, m_propDIOChannel);
}


void CTemplateBlock::SetChannelsInuse()
{
   Engine.IOModList.SetChannelInUse(ANALOG, m_propAIOModule, m_propAIOChannel);
   Engine.IOModList.SetChannelInUse(DIGITAL, m_propDIOModule, m_propDIOChannel);
}


